/******************************************************************************/
/*                                                                            */
/*                Advanced Navigation Packet Protocol Library                 */
/*                Java Language GNSS Compass SDK, Version 7.0                 */
/*                    Copyright 2023, Advanced Navigation                     */
/*                                                                            */
/******************************************************************************/
/*
 * Copyright (C) 2023 Advanced Navigation
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package com.advancednavigation.anPackets;

/* GPIO Output Configuration */
public class ANPacket195
{
	public static final int LENGTH_ = 33;

	public static final int NMEA_FIX_BEHAVIOUR_NORMAL = 0;
	public static final int NMEA_FIX_BEHAVIOUR_ALWAYS_3D = 1;
	public static final int NMEA_FIX_BEHAVIOUR_MAXIMUM = 1;

	public static final String[] OUTPUT_RATE_STRINGS = new String[] {
		"Disabled",
		"0.1 Hz",
		"0.2 Hz",
		"0.5 Hz",
		"1 Hz",
		"2 Hz",
		"5 Hz",
		"10 Hz",
		"25 Hz",
		"50 Hz",
		"8 Hz"
		};
	public static final int RATE_DISABLED = 0;
	public static final int RATE_0o1Hz = 1;
	public static final int RATE_0o2Hz = 2;
	public static final int RATE_0o5Hz = 3;
	public static final int RATE_1Hz = 4;
	public static final int RATE_2Hz = 5;
	public static final int RATE_5Hz = 6;
	public static final int RATE_10Hz = 7;
	public static final int RATE_25Hz = 8;
	public static final int RATE_50Hz = 9;
	public static final int RATE_8Hz = 10;
	public static final int RATE_MAXIMUM = 10;

	public Boolean permanent;

	public Rates gpio1;		// GPIO port
	public Rates gpio3;		// Auxiliary port

	public ANPacket195()
	{
	}

	public ANPacket195(ANPacket packet)
	{
		permanent = packet.data[0] != 0;
		gpio1 = new Rates();
		gpio3 = new Rates();

		gpio3.nmeaFixBehaviour = gpio1.nmeaFixBehaviour = packet.data[1] & 0xFF;
	
		gpio1.gpzda = packet.data[2] & 0x0F;
		gpio3.gpzda = (packet.data[2] & 0xFF) >> 4;
	
		gpio1.gpgga = packet.data[4] & 0x0F;
		gpio3.gpgga = (packet.data[4] & 0xFF) >> 4;
	
		gpio1.gpvtg = packet.data[6] & 0x0F;
		gpio3.gpvtg = (packet.data[6] & 0xFF) >> 4;
	
		gpio1.gprmc = packet.data[8] & 0x0F;
		gpio3.gprmc = (packet.data[8] & 0xFF) >> 4;
	
		gpio1.gphdt = packet.data[10] & 0x0F;
		gpio3.gphdt = (packet.data[10] & 0xFF) >> 4;
	
		gpio1.gpgll = packet.data[12] & 0x0F;
		gpio3.gpgll = (packet.data[12] & 0xFF) >> 4;
	
		gpio1.pashr = packet.data[14] & 0x0F;
		gpio3.pashr = (packet.data[14] & 0xFF) >> 4;
	
		gpio1.tss1 = packet.data[16] & 0x0F;
		gpio3.tss1 = (packet.data[16] & 0xFF) >> 4;
	
		gpio1.simrad = packet.data[18] & 0x0F;
		gpio3.simrad = (packet.data[18] & 0xFF) >> 4;
	
		gpio1.gprot = (packet.data[20] & 0x0F);
		gpio3.gprot = (packet.data[20] & 0xFF) >> 4;
	
		gpio1.gphev = (packet.data[22] & 0x0F);
		gpio3.gphev = (packet.data[22] & 0xFF) >> 4;
	
		gpio1.gpgsv = (packet.data[24] & 0x0F);
		gpio3.gpgsv = (packet.data[24] & 0xFF) >> 4;
	
		gpio1.pfecatt = (packet.data[26] & 0x0F);
		gpio3.pfecatt = (packet.data[26] & 0xFF) >> 4;
	
		gpio1.pfechve = (packet.data[28] & 0x0F);
		gpio3.pfechve = (packet.data[28] & 0xFF) >> 4;
	
		gpio1.pfechve = (packet.data[28] & 0x0F);
		gpio3.pfechve = (packet.data[28] & 0xFF) >> 4;
	
		gpio1.gpgst = (packet.data[30] & 0x0F);
		gpio3.gpgst = (packet.data[30] & 0xFF) >> 4;
	}

	public ANPacket toANPacket() throws IndexOutOfBoundsException
	{
		ANPacket packet;
		if (gpio1 == null || !gpio1.valid() || 
			gpio3 == null || !gpio3.valid())
		{
			throw new IndexOutOfBoundsException();
		}

		packet = new ANPacket(LENGTH_, ANPacket.PACKET_ID_GPIO_OUTPUT_CONFIGURATION);
		packet.data[0] = (byte) (permanent ? 1 : 0);			

		packet.data[1] = (byte) gpio1.nmeaFixBehaviour;
		packet.data[2] = (byte)(gpio1.gpzda | (gpio3.gpzda << 4));
		packet.data[3] = 0;
		packet.data[4] = (byte)(gpio1.gpgga | (gpio3.gpgga<<4));
		packet.data[5] = 0;
		packet.data[6] = (byte)(gpio1.gpvtg | (gpio3.gpvtg<<4));
		packet.data[7] = 0;
		packet.data[8] = (byte)(gpio1.gprmc | (gpio3.gprmc<<4));
		packet.data[9] = 0;
		packet.data[10] = (byte)(gpio1.gphdt | (gpio3.gphdt<<4));
		packet.data[11] = 0;
		packet.data[12] = (byte)(gpio1.gpgll | (gpio3.gpgll<<4));
		packet.data[13] = 0;
		packet.data[14] = (byte)(gpio1.pashr | (gpio3.pashr<<4));
		packet.data[15] = 0;
		packet.data[16] = (byte)(gpio1.tss1 | (gpio3.tss1<<4));
		packet.data[17] = 0;
		packet.data[18] = (byte)(gpio1.simrad | (gpio3.simrad<<4));
		packet.data[19] = 0;
		packet.data[20] = (byte)(gpio1.gprot | (gpio3.gprot<<4));
		packet.data[21] = 0;
		packet.data[22] = (byte)(gpio1.gphev | (gpio3.gphev<<4));
		packet.data[23] = 0;
		packet.data[24] = (byte)(gpio1.gpgsv | (gpio3.gpgsv<<4));
		packet.data[25] = 0;
		packet.data[26] = (byte)(gpio1.pfecatt | (gpio3.pfecatt<<4));
		packet.data[27] = 0;
		packet.data[28] = (byte)(gpio1.pfechve | (gpio3.pfechve<<4));
		packet.data[29] = 0;
		packet.data[30] = (byte)(gpio1.gpgst | (gpio3.gpgst<<4));
		packet.data[31] = 0;
		packet.data[32] = 0;

		return packet;
	}

	public static String rateToString(int rate)
	{
		if(rate < OUTPUT_RATE_STRINGS.length)
		{
			return OUTPUT_RATE_STRINGS[rate];
		}
		else
		{
			return "Unknown Rate";
		}
	}

	public static String nmeaFixBehaviourString(int val)
	{
		if (val == NMEA_FIX_BEHAVIOUR_NORMAL)
		{
			return "Normal";
		}
		else if (val == NMEA_FIX_BEHAVIOUR_ALWAYS_3D)
		{
			return "Always 3D Fix";
		}
		else
		{
			return "Unknown";
		}
	}

	public String nmeaFixBehaviourString()
	{
		if (gpio1 != null)
		{
			return nmeaFixBehaviourString(gpio1.nmeaFixBehaviour);
		}
		else
		{
			return "Unknown";
		}
	}

	public class Rates
	{
		public int nmeaFixBehaviour;
		public int gpzda;
		public int gpgga;
		public int gpvtg;
		public int gprmc;
		public int gphdt;
		public int gpgll;
		public int pashr;
		public int tss1;
		public int simrad;
		public int gprot;
		public int gphev;
		public int gpgsv;
		public int pfecatt;
		public int pfechve;
		public int gpgst;
		boolean valid()
		{
			if (nmeaFixBehaviour < 0 || nmeaFixBehaviour > NMEA_FIX_BEHAVIOUR_MAXIMUM) return false;
			if (gpzda < 0 || gpzda > RATE_MAXIMUM)		return false;
			if (gpgga < 0 || gpgga > RATE_MAXIMUM)		return false;
			if (gpvtg < 0 || gpvtg > RATE_MAXIMUM)		return false;
			if (gprmc < 0 || gprmc > RATE_MAXIMUM)		return false;
			if (gphdt < 0 || gphdt > RATE_MAXIMUM)		return false;
			if (gpgll < 0 || gpgll > RATE_MAXIMUM)		return false;
			if (pashr < 0 || pashr > RATE_MAXIMUM)		return false;
			if (tss1 < 0 || tss1 > RATE_MAXIMUM)		return false;
			if (simrad < 0 || simrad > RATE_MAXIMUM)	return false;
			if (gprot < 0 || gprot > RATE_MAXIMUM)		return false;
			if (gphev < 0 || gphev > RATE_MAXIMUM)		return false;
			if (gpgsv < 0 || gpgsv > RATE_MAXIMUM)		return false;
			if (pfecatt < 0 || pfecatt > RATE_MAXIMUM)	return false;
			if (pfechve < 0 || pfechve > RATE_MAXIMUM)	return false;
			if (gpgst < 0 || gpgst > RATE_MAXIMUM)		return false;

			return true;
		}
	}
}
