/******************************************************************************/
/*                                                                            */
/*                Advanced Navigation Packet Protocol Library                 */
/*                Java Language GNSS Compass SDK, Version 7.0                 */
/*                    Copyright 2023, Advanced Navigation                     */
/*                                                                            */
/******************************************************************************/
/*
 * Copyright (C) 2023 Advanced Navigation
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package com.advancednavigation.anPackets;

/* External Body Velocity */
public class ANPacket47
{
	public static final int LENGTH = 24;

	/* metres per second */
	public float[] velocity;
	public float[] velocityStandardDeviation;
	
	public ANPacket47()
	{
		velocity = new float[3];
		velocityStandardDeviation = new float[3];
	}

	public ANPacket47(ANPacket packet)
	{
		velocity = new float[3];
		velocity[0] = TypeConversion.bytesToFloat(packet.data, 0);
		velocity[1] = TypeConversion.bytesToFloat(packet.data, 4);
		velocity[2] = TypeConversion.bytesToFloat(packet.data, 8);
		velocityStandardDeviation = new float[3];
		velocityStandardDeviation[0] = TypeConversion.bytesToFloat(packet.data, 12);
		if(packet.length == LENGTH)
		{
			velocityStandardDeviation[1] = TypeConversion.bytesToFloat(packet.data, 16);
			velocityStandardDeviation[2] = TypeConversion.bytesToFloat(packet.data, 20);
		}
		else
		{
			velocityStandardDeviation[1] = velocityStandardDeviation[0];
			velocityStandardDeviation[2] = velocityStandardDeviation[0];
		}
	}

	public ANPacket toANPacket()
	{	
		ANPacket packet = new ANPacket(LENGTH, ANPacket.PACKET_ID_EXTERNAL_BODY_VELOCITY);
		TypeConversion.floatToBytes(packet.data, 0, velocity[0]);
		TypeConversion.floatToBytes(packet.data, 4, velocity[1]);
		TypeConversion.floatToBytes(packet.data, 8, velocity[2]);
		TypeConversion.floatToBytes(packet.data, 12, velocityStandardDeviation[0]);
		TypeConversion.floatToBytes(packet.data, 16, velocityStandardDeviation[1]);
		TypeConversion.floatToBytes(packet.data, 20, velocityStandardDeviation[2]);
		return packet;
	}
}
